@@ -2,11 +2,11 @@
 // anyRemote
 // a wi-fi or bluetooth remote for your PC.
 //
-// Copyright (C) 2006-2014 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006-2016 Mikhail Fedotov <anyremote@mail.ru>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
@@ -16,7 +16,7 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
 #include <stdio.h>
@@ -37,12 +37,12 @@
 #define REGEX_KEYVAL   "^[[:space:]]*(\\[[[:alpha:]]+\\])([^=]*)$|^[[:space:]]*([^=%]*[^[:space:]]{1})[[:space:]]*=[[:space:]]*(.*[^[:space:]]{1})[[:space:]]*$"
 #define REGEX_CMDBYCMD "[[:space:]]*(Exec|ExecAndSet|Make|Get|Set|Send|ExecAndSend|Timer|Macro|Load|Include|SendCKPD|Emulate|Dbus)[[:space:]]*(\\()|[[:space:]]*(Exit)[[:space:]]*;{0,1}"
 
-#define REGEX_SET    "[[:space:]]*(bg|caption|editfield|filemanager|fg|font|fullscreen|icons|list|iconlist|menu|parameter|skin|layout|status|text|title|volume|vibrate|image|popup)[[:space:]]*,[[:space:]]*(.*)|[[:space:]]*(repaint|disconnect)[[:space:]]*|[[:space:]]*(cover)[[:space:]]*,[[:space:]]*(by_name|noname)[[:space:]]*,[[:space:]]*(.*)|[[:space:]]*(cover)[[:space:]]*,[[:space:]]*(clear).*"
+#define REGEX_SET    "[[:space:]]*(bg|caption|editfield|filemanager|fg|font|fullscreen|icons|hints|list|iconlist|menu|parameter|skin|layout|status|text|title|volume|vibrate|image|popup)[[:space:]]*,[[:space:]]*(.*)|[[:space:]]*(repaint|disconnect)[[:space:]]*|[[:space:]]*(cover)[[:space:]]*,[[:space:]]*(by_name|noname)[[:space:]]*,[[:space:]]*(.*)|[[:space:]]*(cover)[[:space:]]*,[[:space:]]*(clear).*"
 #define REGEX_GET    "[[:space:]]*(screen_size|cover_size|icon_size|icon_padding|model|version|cursor|ping|password)[[:space:]]*|[[:space:]]*(is_exists)[[:space:]]*,[[:space:]]*(.*)[[:space:]]*,[[:space:]]*(.*)[[:space:]]*|[[:space:]]*(ping)[[:space:]]*,[[:space:]]*(.*)[[:space:]]*"
 #define REGEX_TIMER  "[[:space:]]*([^[:space:]]+)[[:space:]]*,[[:space:]]*([[:digit:]]*|\\$\\(.*\\))[[:space:]]*,[[:space:]]*([[:digit:]]*)[[:space:]]*$|[[:space:]]*([^[:space:]]+)[[:space:]]*,[[:space:]]*(cancel|pause|reset|restart|continue)"
 #define REGEX_MAKE   "[[:space:]]*(remote|mode|var|alarm)[[:space:]]*,[[:space:]]*(.*)[[:space:]]*|[[:space:]]*(exit|flush|stop|disconnect|none|\\$\\(.*\\))[[:space:]]*"
 
-#define REGEX_SET_TL "[[:space:]]*(bg|caption|fg|font|select|icon)[[:space:]]*,(.*)|[[:space:]]*(add|replace)[[:space:]]*,[[:space:]]*([^[:space:]]{1}[^,]*[^[:space:]]{1})[[:space:]]*,[[:space:]]*(.*)|[[:space:]]*(close)[[:space:]]*,[[:space:]]*(clear)[[:space:]]*|[[:space:]]*(clear|close|show|\\$\\(.*\\))[[:space:]]*"
+#define REGEX_SET_TL "[[:space:]]*(bg|caption|fg|font|select|icon)[[:space:]]*,(.*)|[[:space:]]*(add|replace|file|dir)[[:space:]]*,[[:space:]]*([^[:space:]]{1}[^,]*[^[:space:]]{1})[[:space:]]*,[[:space:]]*(.*)|[[:space:]]*(close)[[:space:]]*,[[:space:]]*(clear)[[:space:]]*|[[:space:]]*(clear|close|show|\\$\\(.*\\))[[:space:]]*"
 #define REGEX_SET_MN "[[:space:]]*(add|replace)[[:space:]]*,(.*)|[[:space:]]*(clear)[[:space:]]*"
 #define REGEX_SET_WM "[[:space:]]*(icon|window|cover)[[:space:]]*,(.*)|[[:space:]]*(remove_all|clear_cache|show|close|cursor|nocursor|dynamic_cursor|\\$\\(.*\\))[[:space:]]*|[[:space:]]*(remove|set_cursor),[[:space:]]*(.*)[[:space:]]*"
 #define REGEX_SET_FM "[[:space:]]*(add|replace|select)[[:space:]]*,[[:space:]]*(left|right)[[:space:]]*,[[:space:]]*(.*)|[[:space:]]*(close|show)[[:space:]]*"
@@ -56,27 +56,15 @@ regex_t *keyVal   = NULL;
 regex_t *cmdByCmd = NULL;
 
 CONF conf = 
-{ MODEL_DEFAULT,    // Model
-  NULL,         // Cfg. dir
+{ MODEL_DEFAULT, // Model
   -1,            // Front-end port
-  0,            // set uid
-  0
-};        // set gid
+  0,             // set uid
+  0              // set gid
+};        
 
 static void setCfgDir(char *d)
 {
     //printf("CFG DIR set TO >%s<\n",(d?d:"NULL"));
-    if (conf.cfgDir != NULL) {
-        free(conf.cfgDir);
-        conf.cfgDir = NULL;
-    }
-
-    if (getenv("AR_CFGDIR")) {    // override if set
-        conf.cfgDir = strdup(getenv("AR_CFGDIR"));
-        //printf("CFG DIR >%s<\n",(conf.cfgDir?conf.cfgDir:"NULL"));
-        return;
-    }
-
 
     if (d != NULL) {    // search ../Utils directory
 
@@ -88,26 +76,29 @@ static void setCfgDir(char *d)
 
         if(stat(try2, &buf) == 0) {
             try2[strlen(try2)-6] = '\0';
-            conf.cfgDir = strdup(try2);
-
+            setVarSimple(VAR_CFGDIR, try2);
+            printf("CFG DIR >%s<\n",(try2?try2:"NULL"));
             free(try2);
-            printf("CFG DIR >%s<\n",(conf.cfgDir?conf.cfgDir:"NULL"));
             return;
         }
         free(try2);
     }
 
     // last resort: use {prefix}/anyremote/cfg-data directory
-#ifdef DATADIR
-    conf.cfgDir = (char*) malloc(strlen(DATADIR)+20);    // + "/anyremote/cfg-data"
-    strcpy(conf.cfgDir,DATADIR);
-    strcat(conf.cfgDir,"/anyremote/cfg-data");
-#else
-    conf.cfgDir = (char*) malloc(2);
-    strcpy(conf.cfgDir,".");
-#endif
+    #ifdef DATADIR
+    char* cfgDir = (char*) malloc(strlen(DATADIR)+20);    // + "/anyremote/cfg-data"
+    strcpy(cfgDir,DATADIR);
+    strcat(cfgDir,"/anyremote/cfg-data");
+    #else
+    cfgDir = (char*) malloc(2);
+    strcpy(cfgDir,".");
+    #endif
+    
+    setVarSimple(VAR_CFGDIR, cfgDir);
 
-    //printf("CFG DIR >%s<\n",(conf.cfgDir?conf.cfgDir:"NULL"));
+    //printf("CFG DIR >%s<\n",(cfgDir?cfgDir:"NULL"));
+    free(cfgDir);
+    
 }
 
 /* Parses command line options and set flags. */
@@ -131,6 +122,10 @@ int parse_opts(int argc, char *argv[])
             }
         } else if (strcmp(argv[i],"-password")==0) {
             setUsePassword(BOOL_YES);
+        } else if (strcmp(argv[i],"-cfgdir")==0 && i+1<argc) {
+            setVarSimple("CfgDir", argv[++i]);
+        } else if (strcmp(argv[i],"-tmpdir")==0 && i+1<argc) {
+            setVarSimple("TmpDir", argv[++i]);
         } else if (strcmp(argv[i],"-f")==0) {
             ++i;
             continue;   // already processed this parameter
@@ -300,79 +295,49 @@ static int parseKeyValue(char *in, char ** tag, char ** value)
     return EXIT_OK;
 }
 
+static struct { 
+    const char* name; 
+    int         id;
+} 
+ids[] = {
+    { CMD_MACRO,    ID_MACRO    },   
+    { CMD_MAKE,     ID_MAKE     },   
+    { CMD_EXEC,     ID_EXEC     },   
+    { CMD_SET,      ID_SET      },   
+    { CMD_TIMER,    ID_TIMER    },   
+    { CMD_EXECSET,  ID_EXECSET  },
+    { CMD_GET,      ID_GET      },   
+    { CMD_EXIT,     ID_EXIT     },   
+    { CMD_EMU,      ID_EMU      },   
+    { CMD_DBUS,     ID_DBUS     },   
+    { CMD_INCLUDE,  ID_INCLUDE  },   
+    { CMD_LOAD,     ID_LOAD     },   
+    { CMD_SENDCKPD, ID_SENDCKPD },   
+    { CMD_EXECSEND, ID_EXECSEND },
+    { CMD_SEND,     ID_SEND     },   
+    { NULL,         ID_UNKNOWN  }  
+}; 
+
 const char* id2Cmd (int cmdId)
 {
-    switch(cmdId) {
-    case ID_EXIT:
-        return CMD_EXIT;
-    case ID_EXEC:
-        return CMD_EXEC;
-    case ID_SENDCKPD:
-        return CMD_SENDCKPD;
-    case ID_SET:
-        return CMD_SET;
-    case ID_EXECSET:
-        return CMD_EXECSET;
-    case ID_TIMER:
-        return CMD_TIMER;
-    case ID_SEND:
-        return CMD_SEND;
-    case ID_EXECSEND:
-        return CMD_EXECSEND;
-    case ID_MACRO:
-        return CMD_MACRO;
-    case ID_LOAD:
-        return CMD_LOAD;
-    case ID_INCLUDE:
-        return CMD_INCLUDE;
-    case ID_GET:
-        return CMD_GET;
-    case ID_MAKE:
-        return CMD_MAKE;
-    case ID_EMU:
-        return CMD_EMU;
-    case ID_DBUS:
-        return CMD_DBUS;
-
-    default:
-        return "Unknown";
+    size_t idx;
+    for (idx = 0; ids[idx].name; ++idx) {
+        if (ids[idx].id == cmdId) {
+            break;
+        }
     }
+    return (ids[idx].name ? ids[idx].name : "Unknown");
 }
 
 static int cmd2id(char *name)
 {
-    if (name == NULL) {
-        return ID_UNKNOWN;
-    }
-    
-    static struct { 
-        const char* name; 
-        int         id;
-    } 
-    ids[] = {
-        { CMD_MACRO,    ID_MACRO    },   
-        { CMD_MAKE,     ID_MAKE     },   
-        { CMD_EXEC,     ID_EXEC     },   
-        { CMD_SET,      ID_SET      },   
-        { CMD_TIMER,    ID_TIMER    },   
-        { CMD_EXECSET,  ID_EXECSET  },
-        { CMD_GET,      ID_GET      },   
-        { CMD_EXIT,     ID_EXIT     },   
-        { CMD_EMU,      ID_EMU      },   
-        { CMD_DBUS,     ID_DBUS     },   
-        { CMD_INCLUDE,  ID_INCLUDE  },   
-        { CMD_LOAD,     ID_LOAD     },   
-        { CMD_SENDCKPD, ID_SENDCKPD },   
-        { CMD_EXECSEND, ID_EXECSEND },
-        { CMD_SEND,     ID_SEND     },   
-        { NULL,         ID_UNKNOWN  }  
-    }; 
-    
-    size_t idx;
-    for (idx = 0; ids[idx].name; ++idx) {
-        if (strlen(name) == strlen(ids[idx].name) && 
-            strcmp(name, ids[idx].name) == 0) {
-            return ids[idx].id;
+    if (name) {
+        size_t idx;
+        for (idx = 0; ids[idx].name; ++idx) {
+            if (strlen(name) == strlen(ids[idx].name) && 
+                strcmp(name, ids[idx].name) == 0) {
+                return ids[idx].id;
+            }
         }
     }
     return ID_UNKNOWN;
@@ -397,6 +362,7 @@ int cmdSet2id(const char *name)
         { SET_FONT,    ID_SET_FONT   },  
         { SET_FSCREEN, ID_SET_FSCREEN},  
         { SET_ICONS,   ID_SET_ICONS  },  
+        { SET_HINTS,   ID_SET_HINTS  },  
         { SET_LIST,    ID_SET_LIST   },  
         { SET_ILIST,   ID_SET_ILIST  }, 
         { SET_MENU,    ID_SET_MENU   },   
@@ -438,17 +404,17 @@ int cmdGet2id(const char *name)
         int         id;
     } 
     gids[] = {
-    { GET_SCREENSIZE, ID_GET_SCREENSIZE },
-    { GET_COVERSIZE,  ID_GET_COVERSIZE  },  
-    { GET_ICONSIZE,   ID_GET_ICONSIZE   },  
-    { GET_ICONPADDING,ID_GET_ICONPADDING},
-    { GET_MODEL,      ID_GET_MODEL      },      
-    { GET_VERSION,    ID_GET_VERSION    },    
-    { GET_CURSOR,     ID_GET_CURSOR     },     
-    { GET_PING,       ID_GET_PING       },       
-    { GET_PASSWORD,   ID_GET_PASSWORD   },   
-    { GET_ISEXISTS,   ID_GET_ISEXISTS   },
-    { NULL,           ID_GET_MAX        }
+        { GET_SCREENSIZE, ID_GET_SCREENSIZE },
+        { GET_COVERSIZE,  ID_GET_COVERSIZE  },  
+        { GET_ICONSIZE,   ID_GET_ICONSIZE   },  
+        { GET_ICONPADDING,ID_GET_ICONPADDING},
+        { GET_MODEL,      ID_GET_MODEL      },      
+        { GET_VERSION,    ID_GET_VERSION    },    
+        { GET_CURSOR,     ID_GET_CURSOR     },     
+        { GET_PING,       ID_GET_PING       },       
+        { GET_PASSWORD,   ID_GET_PASSWORD   },   
+        { GET_ISEXISTS,   ID_GET_ISEXISTS   },
+        { NULL,           ID_GET_MAX        }
     };
     
     size_t idx;
@@ -837,6 +803,7 @@ static int storeExecAndSetCmdEx(cmdItem* ci, char *cmd, int execFlag)
                 stype == ID_SET_FONT    ||
                 stype == ID_SET_FSCREEN ||
                 stype == ID_SET_ICONS   ||
+                stype == ID_SET_HINTS   ||
                 stype == ID_SET_LAYOUT  ||
                 stype == ID_SET_STATUS  ||
                 stype == ID_SET_TITLE   ||
@@ -974,7 +941,7 @@ static int storeExecAndSetCmdEx(cmdItem* ci, char *cmd, int execFlag)
                             strncat(ci->descr,start2 + result2[2].rm_so, le);
                         }
 
-                    } else if (result2[3].rm_so >= 0) {    // add|replace,_title_
+                    } else if (result2[3].rm_so >= 0) {    // add|replace|file|dir,_title_
 
                         l2 = result2[3].rm_eo - result2[3].rm_so;
                         int l3 = result2[4].rm_eo - result2[4].rm_so;
@@ -1149,7 +1116,7 @@ static int storeExecAndSetCmdEx(cmdItem* ci, char *cmd, int execFlag)
             int l4 = result[4].rm_eo - result[4].rm_so;
             int l5 = result[5].rm_eo - result[5].rm_so;
             int l6 = result[6].rm_eo - result[6].rm_so;
-	    //printf("storeExecAndSetCmdEx(): cover %d %d %d\n",l4,l5,l6);
+            //printf("storeExecAndSetCmdEx(): cover %d %d %d\n",l4,l5,l6);
 
             ci->subtype = ID_SET_COVER;
             
@@ -1176,7 +1143,7 @@ static int storeExecAndSetCmdEx(cmdItem* ci, char *cmd, int execFlag)
             int l8 = result[8].rm_eo - result[8].rm_so;
             ci->subtype = ID_SET_COVER;
             
-	    //printf("storeExecAndSetCmdEx(): coverCLEAR %d %d \n",l7,l8);
+            //printf("storeExecAndSetCmdEx(): coverCLEAR %d %d \n",l7,l8);
 	     
             ci->descr = (char*) calloc(1,l7+l8+2);
             strncpy(ci->descr,cmd+result[7].rm_so,l7);
@@ -1187,7 +1154,7 @@ static int storeExecAndSetCmdEx(cmdItem* ci, char *cmd, int execFlag)
             printf("storeExecAndSetCmdEx(): parse error (1)\n");
             parseFail = 1;
         }
-	//printf("storeExecAndSetCmdEx(): parsed.\n");
+        //printf("storeExecAndSetCmdEx(): parsed.\n");
     } else {
         // ExecAndSet(_shell_commands_) goes here ????
         printf("storeExecAndSetCmdEx(): parse error (2)\n");
@@ -1781,18 +1748,23 @@ void freeRegexps()
 
 void init_cfg_dir(char *path)
 {
-    if (path != NULL) {
-        char *d = strdup(path);
-        char *p = rindex(d,SEPARATOR);
-        if (p != NULL) {
-            *p = '\0';
+    int sz = 0;
+    const char* v = getVarValue("CfgDir", &sz);
+    
+    if (!v) {
+        if (path != NULL) {
+            char *d = strdup(path);
+            char *p = rindex(d,SEPARATOR);
+            if (p != NULL) {
+                *p = '\0';
+            } else {
+                *d = '\0';
+            }
+            setCfgDir(d);
+            free(d);
         } else {
-            *d = '\0';
+            setCfgDir(NULL);
         }
-        setCfgDir(d);
-        free(d);
-    } else {
-        setCfgDir(NULL);
     }
 }
 
